<?php
/**
* LICENCIA
*
* Este programa se propociona "tal cual", sin garantía de ningún tipo más allá del soporte
* pactado a la hora de adquirir el programa.
*
* En ningún caso los autores o titulares del copyright serán responsables de ninguna
* reclamación, daños u otras responsabilidades, ya sea en un litigio, agravio o de otro
* modo, que surja de o en conexión con el programa o el uso u otro tipo de acciones
* realizadas con el programa.
*
* Este programa no puede modificarse ni distribuirse sin el consentimiento expreso del autor.
*
*    @author    Carlos Fillol Sendra <festeweb@festeweb.com>
*    @copyright 2014 Fes-te web! - www.festeweb.com
*    @license   http://www.festeweb.com/static/licenses/fs2ps_1.1.0.txt
*/

include_once(dirname(__FILE__).'/Fs2psObjectModels.php');

class Fs2psDto2RowMatcher
{
    public $entity;
	public $table;
	public $dto_id_fields;
	public $row_id_field;
	public $persist;
	public $forced_maches;
	public $direct_match;

	public function __construct($entity, $table, $row_id_field=null, $dto_id_fields=null, $persist = true)
	{
		$this->entity = $entity;
		$this->table = $table;
		$this->row_id_field = $row_id_field? $row_id_field : Fs2psObjectModel::idForTable($table);
		$this->dto_id_fields = $dto_id_fields? $dto_id_fields : array('ref');
		$this->persist = $persist;
	}
	
	public function reloadCfg($cfg) {
	    $updpart = strtoupper($this->entity);
	    $this->forced_maches = $cfg->get('MATCH_'.$updpart, array());
	    $this->direct_match = $cfg->get('DIRECT_MATCH_'.$updpart, false);
	}

	
	public function dtoIdToStr($dto_id)
	{
	    if (is_array($dto_id))
	    {
	        $dto_id_array = array();
	        foreach ($dto_id as $id_part)
	        {
	            $id_part = empty($id_part) && $id_part!=='0' ? '' : $id_part; // empty de string '0' se evalúa a true!!!?? 
	            
	            // Escape id_part separator in each id_part: '_' -> '#_'
	            $id_part = str_replace('#', '##', $id_part);
	            $id_part = str_replace('_', '#_', $id_part);
	            
	            $dto_id_array[] = $id_part;
	        }
	        return implode('_', $dto_id_array);
	    }
	    return (string)$dto_id;
	}
	
	public function strToDtoId($str)
	{
	    if (empty($str) && $str!=='0') return ''; // empty de string '0' se evalúa a true!!!??
	    
	    $str = $str.'_';
	    $dto_id_array = array();
	    
	    $matches = null;
	    preg_match('/^(?:((?:(?:#_)|(?:##)|[^_])*)_)/', $str, $matches);
	    while (!empty($matches))
	    {
	        $id_part = $matches[1];
	        
	        // Go to next match
	        $str = substr($str, strlen($id_part) + 1);
	        preg_match('/^(?:((?:(?:#_)|(?:##)|[^_])*)_)/', $str, $matches);
	        
	        // Reverse _ and # escaping
	        $id_part = str_replace('#_', '_', $id_part);
	        $id_part = str_replace('##', '#', $id_part);
	        
	        $dto_id_array[] = $id_part;
	    }
	    
	    if (empty($dto_id_array)) return null;
	    return count($dto_id_array) > 1? $dto_id_array : $dto_id_array[0];
	}

	public function dtoIdToStrFromDto($dto)
	{
	    return $this->dtoIdToStr($this->dtoId($dto));
	}
	
	public function dtoId($dto)
	{
	    if (isset($dto['_id'])) return $dto['_id'];
	        
        $id_as_array = array();
        foreach ($this->dto_id_fields as $f)
        {
            $value = $dto[$f];
            $value = empty($value) && $value!=='0' ? null : $value; // empty de string '0' se evalúa a true!!!??
            array_push($id_as_array, $value);
        }
        $id = count($id_as_array) > 1 ? $id_as_array : $id_as_array[0];
        $dto['_id'] = $id;
        return $id;
	}
	
	public function referenceFromDto($dto)
	{
	    return isset($dto['pref'])? $dto['pref'] : $this->dtoIdToStrFromDto($dto);
	}
	
	public function rowId($row)
	{
	    if (isset($row[$this->row_id_field])) {
	        return $row[$this->row_id_field];
	    }
	    return null;
	}
	
	
	
	protected function getMatch($dto_id)
	{
		$dto_id_str = Fs2psTools::dbEscape($this->dtoIdToStr($dto_id));
		$row_id = Fs2psTools::dbValue('
			select row_id
			from `@DB_fs2ps_match`
			where 
				`table`=\''.$this->table.'\' and
				(`entity`=\''.$this->entity.'\' or `entity` is null) and
				dto_id=\''.$dto_id_str.'\'
		');

		if ($row_id)
			return $row_id;
		return null;
	}
	
	protected function getReverseMatch($row_id)
	{
		$dto_id_str = Fs2psTools::dbValue('
			select dto_id
			from `@DB_fs2ps_match`
			where
				`table`=\''.$this->table.'\' and
				(`entity`=\''.$this->entity.'\' or `entity` is null) and
				row_id=\''.$row_id.'\'
		');
	
		if ($dto_id_str)
			return $dto_id_str;
		return null;
	}
	
	public function existsRowInDatabase($id)
	{
	    return $id!==null && $id!=='' && Fs2psObjectModel::existsInDatabase($id, $this->table); // id puede ser 0
	}
	
	
	public function rowIdFromDtoIdForcedMatches($dto_id)
	{
	    if ($this->direct_match) return $dto_id;
	    if (!empty($this->forced_maches))
	    {
	        $dto_id_str = $this->dtoIdToStr($dto_id);
	        if (isset($this->forced_maches[$dto_id_str]))
	            return $this->forced_maches[$dto_id_str];
	    }
	    return null;
	}
	
	public function rowIdFromDtoId($dto_id)
	{
	    $row_id = $this->rowIdFromDtoIdForcedMatches($dto_id);
	    if ($row_id) return $row_id;
	    return $this->_rowIdFromDtoId($dto_id);
	}
	
	/**
	 * Esta es la función que debe sobreescribirse para modificar el comportamiento del
	 * matcher.
	 */
	public function _rowIdFromDtoId($dto_id)
	{
	    if (!$this->persist) return null;
	    return $this->getMatch($dto_id);
	}
	
	/**
	 * Sólo debe usarse esta función en caso de que dto tenga suficiente información
	 * como en el momento de la creación/actualización.
	 */
	public function rowIdFromDto($dto)
	{
	    return $this->rowIdFromDtoId($this->dtoId($dto));
	}
	
	public function dtoIdStrFromRowId($row_id)
	{
		if (!$this->persist) return null;
		return $this->getReverseMatch($row_id);
	}

	public function updateMatch($dto_id, $row_id)
	{
		if (!$this->persist) return;

		$dto_id_str = Fs2psTools::dbEscape($this->dtoIdToStr($dto_id));
		if ($this->getMatch($dto_id))
		{
			return Fs2psTools::dbExec('
				update `@DB_fs2ps_match`
				set row_id='.$row_id.', entity=\''.$this->entity.'\', uploaded=1
				where 
					`table`=\''.$this->table.'\' and
					(`entity`=\''.$this->entity.'\' or `entity` is null) and
					dto_id=\''.$dto_id_str.'\'
			');
		}
		else
		{
			return Fs2psTools::dbExec('
				insert into `@DB_fs2ps_match` (`table`, `entity`, dto_id, row_id, uploaded)
				values (\''.$this->table.'\', \''.$this->entity.'\', \''.$dto_id_str.'\', '.$row_id.', 1)
			');
		}
	}

	public static function clearUploadedMarks()
	{
		// Eliminamos refs. de match si se eliminan datos en PS
		$rows = Fs2psTools::dbSelect(
			'select distinct `table`, `entity` from `@DB_fs2ps_match`'
		);
		foreach ($rows as $row)
		{
			$table = $row['table'];
			$entity = $row['entity'];
			$id_field = Fs2psObjectModel::idForTable($table);
			$where = '';
			
			// Purga de matches que han dejado de existir
			Fs2psTools::dbExec('
				delete from @DB_fs2ps_match
				where 
					`table`=\''.$table.'\' and 
					(`entity`=\''.$entity.'\' or `entity`is null) and 
					row_id not in (
						select '.$id_field.' from @DB_'.$table.' '.$where.'
					)
			');
		}
		
		// Reseteamos marca uploaded
		Fs2psTools::dbExec('
			update `@DB_fs2ps_match`
			set uploaded = 0
		');
	}
	
	
	/**
	 *  Eliminamos refs. de match duplicadas de las cuales sólo una tiene uploaded=1
	 *  Esto puede pasar si cambia el valor de la referencia en Prestashop o la política de matching.
	 *  Ej:   dto_id      row_id       uploaded
	 *           001          11              0  <- Esta se debe borrar
	 *           003          11              1  <- Esta se debe conservar
	 *           005          11              0  <- Esta se debe borrar
	 */
	public static function deleteRepeatedZeroMarks()
	{
	    Fs2psTools::dbExec('
			delete  r0
			from
			    @DB_fs2ps_match r1
			    inner join @DB_fs2ps_match r0 on r0.`table`=r1.`table` and r0.entity=r1.entity and r0.row_id=r1.row_id and r0.uploaded=0
			where r1.uploaded=1
		');
	}

}
