<?php
/**
* LICENCIA
*
* Este programa se propociona "tal cual", sin garantía de ningún tipo más allá del soporte
* pactado a la hora de adquirir el programa.
*
* En ningún caso los autores o titulares del copyright serán responsables de ninguna
* reclamación, daños u otras responsabilidades, ya sea en un litigio, agravio o de otro
* modo, que surja de o en conexión con el programa o el uso u otro tipo de acciones
* realizadas con el programa.
*
* Este programa no puede modificarse ni distribuirse sin el consentimiento expreso del autor.
*
*    @author    Carlos Fillol Sendra <festeweb@festeweb.com>
*    @copyright 2014 Fes-te web! - www.festeweb.com
*    @license   http://www.festeweb.com/static/licenses/fs2ps_1.1.0.txt
*/

include_once(dirname(__FILE__).'/Fs2psTools.php');
include_once(dirname(__FILE__).'/Fs2psExtractor.php');
include_once(dirname(__FILE__).'/Fs2psMatchers.php');
include_once(dirname(__FILE__).'/Fs2psObjectModels.php');
include_once(dirname(__FILE__).'/Fs2psUpdaters.php');


class Fs2psOrderDependentExtractor extends Fs2psExtractor
{
	protected $download_orders;
	protected $ifvalid;

	protected function reloadCfg() {
		parent::reloadCfg();
		$cfg = $this->task->cfg;
		$this->download_orders = $cfg->get('DOWNLOAD_ORDERS', '');
		$this->ifvalid = strpos($this->download_orders, 'ifvalid') !== false;
	}

	protected function getAfterDateWhereCondition()
	{
		if (empty($this->task->cmd['after']))
			$dto_after = '1970-01-01 01:00:00';
		else
			$dto_after = $this->task->cmd['after'];

		$after =  Fs2psTools::dto2date($dto_after);
		$after_str = Fs2psTools::date2db($after);
		
		$download_orders_where = ' and o.post_status not in (\'wc-cancelled\', \'wc-failed\', \'wc-pending\') ';
		// TODO: if ($this->ifvalid) $download_orders_where = ' and o.post_status not in (\'wc-cancelled\', \'wc-failed\') ';
		
		return 'o.post_modified>\''.$after_str.'\' and o.post_type=\'shop_order\' '.$download_orders_where;
	}
}

class Fs2psCustomerExtractor extends Fs2psOrderDependentExtractor
{
	
	protected $download_customers;
	protected $noorder;
	protected $noaddress;
	protected $optional_fields = array('address2');
	
	protected function reloadCfg() {
		parent::reloadCfg();
		$cfg = $this->task->cfg;
		$this->download_customers = $cfg->get('DOWNLOAD_CUSTOMERS', '');
		$this->noaddress = strpos($this->download_customers, 'noaddress') !== false;
		$this->noorder = $this->noaddress || strpos($this->download_customers, 'noorder') !== false;
	}
	
	protected function buildSql()
	{
		// XXX cfillol: Cuidado con los selects con columnas que no estan en el group by
		// su soporte puede estar deshabilitado en algunos MySQLs y el orden puede ser arbitrario.
		// https://dev.mysql.com/doc/refman/5.0/en/group-by-handling.html
		// http://renormalize.org/post/24114561260/mysql-curious-features-non-deterministic-group-by
		
		// XXX cfillol: ¿Qué pasa si hay varias direcciones del mismo cliente con el mismo nif?
		// Puede que no se cargue la correcta, sobretodo si se activa 'noorder'
		
		/* No enviamos información acerca de la dirección de envío
	 		,(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_address_1\') as saddress1,  
	        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_address_2\') as saddress2,  
	        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_postcode\') as spostcode,  
	        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_city\') as scity, 
			(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_phone\') as sphone,
			\'\' as sphone_mobile,
	        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_state\') as sstate,  
	        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_country\') as scountry
		 */
		
		return '
			SELECT 
				o.post_date as date_add,
				o.post_modified as date_upd,
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_first_name\') as firstname,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_last_name\') as lastname,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_email\') as email, 
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_company\') as company,
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_cif_dni\') as dni,
				\'\' as vat_number,
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_address_1\') as address1,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_address_2\') as address2,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_postcode\') as postcode,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_city\') as city, 
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_phone\') as phone,
				\'\' as phone_mobile,
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_state\') as state,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_country\') as country
	        FROM               
	            @DB_posts o
				inner join @DB_postmeta om on om.post_id=o.ID and om.meta_key=\'_billing_email\'
	            -- inner join @DB_postmeta om on om.post_id=o.ID and om.meta_key=\'_customer_user\'
	            -- inner join @DB_users c on c.ID=om.meta_value
	        WHERE 
	            '.$this->getAfterDateWhereCondition().' 
	       	GROUP BY om.meta_value
	        ORDER BY min(o.post_modified), om.meta_value
		';
	}

	protected function row2dto($row)
	{
		
		$dto = array(
			'user' => $row['email'],
			'created' => $row['date_add'],
			'updated' => $row['date_upd'],
			'company' => $row['company'],
			'nif' => $row['dni'],
			'vat_number' => '', // $row['vat_number'], 
			'firstname' => $row['firstname'],
			'lastname' => $row['lastname'],
			'email' => $row['email'],
			'address' => $row['address1'],
			'postcode' => $row['postcode'],
			'city' => $row['city'],
			'state' => WC()->countries->states[$row['country']][$row['state']],
			'country' => WC()->countries->countries[$row['country']],
			'phone' => $row['phone'],
			'mobile' => '',
		);
		
		foreach ($this->optional_fields as $of) {
		    if (!empty($row[$of])) $dto[$of] = $row[$of];
		}
		
		return $dto;
	}

}


class Fs2psOrderExtractor extends Fs2psOrderDependentExtractor
{
    
  protected $optional_fields = array('address2', 'saddress2');
	
	protected function buildSql()
	{
		
		/*
		_cart_discount	16.97			Base del escuento total
		_cart_discount_tax	3.56		IVA del descuento total
		_order_shipping	3.95			Base del transporte
		_order_shipping_tax	0			IVA del transporte
		_order_tax	32.08				IVA total del pedido
		_order_total	188.77			Total del pedido (lo que se paga, con IVA y descuentos)
		 */
		
		return '
			SELECT 
				o.ID as id_order,
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_order_key\') as reference,
				null as invoice_number,
				o.post_date as date_add,
				o.post_modified as date_upd,
				o.post_excerpt as note,
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_cart_discount\') as _cart_discount,
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_cart_discount_tax\') as _cart_discount_tax,
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_order_shipping\') as _order_shipping,
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_order_shipping_tax\') as _order_shipping_tax,
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_order_tax\') as _order_tax,
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_order_total\') as _order_total,
				1 as valid,
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_first_name\') as firstname,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_last_name\') as lastname,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_email\') as email, 
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_company\') as company,
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_cif_dni\') as dni,
				\'\' as vat_number,
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_address_1\') as address1,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_address_2\') as address2,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_postcode\') as postcode,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_city\') as city, 
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_phone\') as phone,
				\'\' as phone_mobile,
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_state\') as state,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_billing_country\') as country,
				
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_address_1\') as saddress1,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_address_2\') as saddress2,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_postcode\') as spostcode,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_city\') as scity, 
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_phone\') as sphone,
				\'\' as sphone_mobile,
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_state\') as sstate,  
		        (select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_shipping_country\') as scountry,
				
				(select meta_value from @DB_postmeta where post_id=o.ID and meta_key=\'_payment_method\') as payment
				
	        FROM               
	            @DB_posts o
				inner join @DB_postmeta om on om.post_id=o.ID and om.meta_key=\'_billing_email\'
	            -- inner join @DB_postmeta om on om.post_id=o.ID and om.meta_key=\'_customer_user\'
	            -- inner join @DB_users c on c.ID=om.meta_value
	        WHERE 
	            '.$this->getAfterDateWhereCondition().' 
	        ORDER BY o.post_modified, om.meta_value
		';
	}
	
	protected function getOrderTotals($row)
	{
		$totals_rows = Fs2psTools::dbSelect('
			SELECT 
				tr.tax_rate as tax, 
				sum(oim_line_total.meta_value) as line_base,
				sum(oim_line_tax.meta_value) as line_tax
			FROM 
			    @DB_posts o
			    -- lineas de producto
			    inner join @DB_woocommerce_order_items oi_lines on oi_lines.order_id=o.ID and oi_lines.order_item_type=\'line_item\'
			    inner join @DB_woocommerce_order_itemmeta oim_tax_class on oim_tax_class.order_item_id=oi_lines.order_item_id and oim_tax_class.meta_key=\'_tax_class\'
			    inner join @DB_woocommerce_order_itemmeta oim_line_tax on oim_line_tax.order_item_id=oi_lines.order_item_id and oim_line_tax.meta_key=\'_line_tax\'
			    inner join @DB_woocommerce_order_itemmeta oim_line_total on oim_line_total.order_item_id=oi_lines.order_item_id and oim_line_total.meta_key=\'_line_total\'
			    -- impuestos
			    inner join @DB_woocommerce_order_items oi_rate_id on oi_rate_id.order_id=o.ID and oi_rate_id.order_item_type=\'tax\'
			    inner join @DB_woocommerce_order_itemmeta oim_rate_id on oim_rate_id.order_item_id=oi_rate_id.order_item_id and oim_rate_id.meta_key=\'rate_id\'   
			    -- Enlazamos tax_class con correpondiente tax_rate
			    inner join @DB_woocommerce_tax_rates tr on tr.tax_rate_class=oim_tax_class.meta_value and tr.tax_rate_id=oim_rate_id.meta_value
			where
			    o.ID='.$row['id_order'].'
			GROUP BY tr.tax_rate
			ORDER BY tr.tax_rate
		');

		$dec = 1000; // decimales de precisión para impuestos
		
		$taxes_by_rate = array();
		$lines_tax_amounts_by_tax = array();
		$lines_bases_by_tax = array();
		$discount_bases_by_tax = array();
		$shipping_tax_amounts_by_tax = array();
		$shipping_bases_by_tax = array();
		
		$carrier_tax_rate = null;
		if ($row['total_shipping_tax_excl']>0) {
    		$carrier_tax_rate = floatval($row['carrier_tax_rate'])*100.0;
    		$carrier_tax_rate_key = intval($carrier_tax_rate * $dec);
    		$taxes_by_rate[$carrier_tax_rate_key] = $carrier_tax_rate;
    		$lines_tax_amounts_by_tax[$carrier_tax_rate_key] = 0;
    		$lines_bases_by_tax[$carrier_tax_rate_key] = 0;
    		$discount_bases_by_tax[$carrier_tax_rate_key] = 0;
    		$shipping_tax_amounts_by_tax[$carrier_tax_rate_key] = 0;
    		$shipping_bases_by_tax[$carrier_tax_rate_key] = 0;
		}
		
		$total_products_base = floatval($row['total_products']);
		$discount_ratio = $total_products_base>0? ($total_products_base-floatval($row['total_discounts_tax_excl']))/$total_products_base : 1;
		foreach ($totals_rows as $totals_row)
		{
			$rate = floatval($totals_row['tax']);
			$rate_key = intval($rate * $dec);
			$taxes_by_rate[$rate_key] = $rate;
			$lines_bases_by_tax[$rate_key] = floatval($totals_row['line_base']);
			$lines_tax_amounts_by_tax[$rate_key] = floatval($totals_row['line_tax']);
			$discount_bases_by_tax[$rate_key] = Fs2psTools::ps_round($lines_bases_by_tax[$rate_key] * (1 - $discount_ratio), 2);
			$discount_bases_by_tax[$rate] = 0;
			
			// Nos aseguramos de que estén inicializadas a 0 otras tasas distintas a la del transporte
			$shipping_tax_amounts_by_tax[$rate_key] = 0; 
			$shipping_bases_by_tax[$rate_key] = 0;
		}
		if (!is_null($carrier_tax_rate)) {
    		$shipping_tax_amounts_by_tax[$carrier_tax_rate_key] = floatval($row['total_shipping_tax_incl']) - floatval($row['total_shipping_tax_excl']);
    		$shipping_bases_by_tax[$carrier_tax_rate_key] = floatval($row['total_shipping_tax_excl']);
		}
	
		$taxes = array();
		$lines_tax_amounts = array();
		$lines_bases = array();
		$discount_bases = array();
		$shipping_tax_amounts = array();
		$shipping_bases = array();
		foreach ($taxes_by_rate as $rate_key => $val)
		{
		    $taxes[] = $rate_key/$dec;
			$lines_tax_amounts[] = $lines_tax_amounts_by_tax[$rate_key];
			$lines_bases[] = $lines_bases_by_tax[$rate_key];
			$discount_bases[] = $discount_bases_by_tax[$rate_key];
			$shipping_tax_amounts[] = $shipping_tax_amounts_by_tax[$rate_key];
			$shipping_bases[] = $shipping_bases_by_tax[$rate_key];
		}
	
		// Correcciones contables. ¿Que sucede si hay descuadres?
		$total = floatval($row['total_paid_tax_incl']);
		$total_calc = (
		    (array_sum($lines_bases) + array_sum($shipping_bases)) - array_sum($discount_bases) +
		    (array_sum($lines_tax_amounts) + array_sum($shipping_tax_amounts))
		);
		$error = round($total_calc - $total, 2); // redondeamos xq a veces se generan periodos aunque sean sumas y restas :/
		if ($error>0) {
			// Si el descuadre es positivo, lo descontamos del total.
			// Nota: Esto puede provocar que se pague más IVA del necesario.
			foreach ($lines_bases as $i => $base) {
				if (($base-$discount_bases[$i]) >= $error) {
					$discount_bases[$i] += $error;
					break;
				}
			}
		}
		else if ($error<0) {
			// Descuadres en Prestashop?! Puede que el usuario pagara más de lo debido.
			// No debería suceder a menos que exista un bug en los cálculos en Prestashop (ej: PS 1.6.0.9)
			// o exista un bug en el conector ...
			$total = $total + $error;
		}
	
		return array(
			'taxes' => $taxes,
			'lines_tax_amounts' => $lines_tax_amounts,
			'lines_bases' => $lines_bases,
			'discount_bases' => $discount_bases,
			'shipping_tax_amounts' => $shipping_tax_amounts,
			'shipping_bases' => $shipping_bases,
			'total' => $total
		);
	}
	
	protected function row2dto($row)
	{
		
		$row['total_products'] = $row['_order_total'] - $row['_order_tax'] - $row['_order_shipping'];
		$row['total_paid_tax_incl'] = $row['_order_total'];
		$row['carrier_tax_rate'] = $row['_order_shipping_tax']>0? Fs2psTools::ps_round($row['_order_shipping_tax']/$row['_order_shipping'], 2) : 0.0;
		$row['total_shipping_tax_incl'] = $row['_order_shipping_tax'] + $row['_order_shipping'];
		$row['total_shipping_tax_excl'] = $row['_order_shipping'];
		$row['total_discounts_tax_excl'] = $row['_cart_discount'];
		
		$totals = $this->getOrderTotals($row);
					
		$dto = array(
			'id' => intval($row['id_order']),
			'ref' => $row['reference'],
			'invoice' => $row['invoice_number'],
			//'invoice' => intval($row['id_order_invoice']),
			'created' => $row['date_add'],
			'updated' => $row['date_upd'],
			'taxes' => $totals['taxes'],
			'discount_bases' => $totals['discount_bases'],
			'lines_tax_amounts' => $totals['lines_tax_amounts'],
			'lines_bases' => $totals['lines_bases'],
			'shipping_tax_amounts' => $totals['shipping_tax_amounts'],
			'shipping_bases' => $totals['shipping_bases'],
			'total' => $totals['total'],
				
			'company' => empty($row['company'])? $row['firstname'].' '.$row['lastname'] : $row['company'],
			'user' => $row['email'],
			'nif' => $row['dni'],
			'vat_number' => $row['vat_number'],
				
			'address' => $row['address1'],
			'postcode' => $row['postcode'],
			'city' => $row['city'],
			'state' => WC()->countries->states[$row['country']][$row['state']],
			'country' => WC()->countries->countries[$row['country']],
			'phone' => $row['phone'],
			'mobile' => $row['phone_mobile'],
			'email' => $row['email'],
				
			'saddress' => $row['saddress1'],
			'spostcode' => $row['spostcode'],
			'scity' => $row['scity'],
			'sstate' => WC()->countries->states[$row['scountry']][$row['sstate']],
			'scountry' => WC()->countries->countries[$row['scountry']],
			'sphone' => $row['sphone'],
			'smobile' => $row['sphone_mobile'],
				
			'valid' => intval($row['valid'])==1,
			'payment' => $row['payment'],
		);
		
		foreach ($this->optional_fields as $of) {
            if (!empty($row[$of])) $dto[$of] = $row[$of];
		}

		if (!empty($row['note'])) {
			$dto['note'] = $row['note'];
		}
		
		return $dto;
	}
}


class Fs2psOrderLineExtractor extends Fs2psOrderDependentExtractor
{

    protected $productDto2RowMatcher;
    
    public function __construct($task, $name)
    {
        parent::__construct($task, $name);
        $this->productDto2RowMatcher = Fs2psProductUpdater::createMatcher($task, 'products');
    }
    
    
	protected function buildSql()
	{
		
		return '
			SELECT 
				o.ID as id_order, 
				null as id_order_invoice,
				(   
                    select meta_value from @DB_postmeta
                    where post_id = (
                        select meta_value from @DB_woocommerce_order_itemmeta
                        where order_item_id=oi.order_item_id and meta_key=\'_product_id\'   
                    ) and meta_key=\'_sku\'                            
                ) as product_reference,
                oi.order_item_id as id_product,
                (select meta_value from @DB_woocommerce_order_itemmeta m where m.order_item_id=oi.order_item_id and (m.meta_key like \'%talla%\' or m.meta_key like \'%size%\')) as size,
                (select meta_value from @DB_woocommerce_order_itemmeta m where m.order_item_id=oi.order_item_id and m.meta_key like \'%colo%\') as color,
				oi.order_item_name as product_name,
				(select meta_value from @DB_woocommerce_order_itemmeta m where m.order_item_id=oi.order_item_id and m.meta_key=\'_qty\') as quantity,
				tr.tax_rate as rate, 
                (select meta_value from @DB_woocommerce_order_itemmeta m where m.order_item_id=oi.order_item_id and m.meta_key=\'_line_subtotal\') as line_subtotal,
                (select meta_value from @DB_woocommerce_order_itemmeta m where m.order_item_id=oi.order_item_id and m.meta_key=\'_line_subtotal_tax\') as line_subtotal_tax,
                (select meta_value from @DB_woocommerce_order_itemmeta m where m.order_item_id=oi.order_item_id and m.meta_key=\'_line_total\') as line_total,
                (select meta_value from @DB_woocommerce_order_itemmeta m where m.order_item_id=oi.order_item_id and m.meta_key=\'_line_tax\') as line_tax
		
			FROM 
			    @DB_posts o
			    -- lineas de producto
			    inner join @DB_woocommerce_order_items oi on oi.order_id=o.ID and oi.order_item_type=\'line_item\'
			    inner join @DB_woocommerce_order_itemmeta oim_tax_class on oim_tax_class.order_item_id=oi.order_item_id and oim_tax_class.meta_key=\'_tax_class\'
			    -- impuestos
			    inner join @DB_woocommerce_order_items oi_rate_id on oi_rate_id.order_id=o.ID and oi_rate_id.order_item_type=\'tax\'
			    inner join @DB_woocommerce_order_itemmeta oim_rate_id on oim_rate_id.order_item_id=oi_rate_id.order_item_id and oim_rate_id.meta_key=\'rate_id\'   
			    -- Enlazamos tax_class con correpondiente tax_rate
			    inner join @DB_woocommerce_tax_rates tr on tr.tax_rate_class=oim_tax_class.meta_value and tr.tax_rate_id=oim_rate_id.meta_value						
			WHERE
				'.$this->getAfterDateWhereCondition().'
			ORDER BY o.post_modified, o.ID, oi.order_item_id
		';
	}
	
	protected function row2dto($row)
	{
        $pm = $this->productDto2RowMatcher;
        $ref = $pm->persist ? $pm->dtoIdStrFromRowId($row['id_product']) : $row['product_reference'];
        if ($pm->persist && ! $ref) $ref = $row['product_reference']; // Puede que queden por descargar pedidos anteriores al cambio de matcher
	    
		/*
		 _product_id	4516
		_variation_id		
		_qty	3
		tallas	S/M
		_line_subtotal	89.1322			Base de linea antes del descuento
		_line_subtotal_tax	18.7178		IVA de linea antes del descuento
		_line_total	80.2231				Base de linea después del descuento
		_line_tax	16.8469				IVA de linea después del descuento
		*/
		
		$qty = floatval($row['quantity']);
		$tax = floatval($row['rate']);
		
		$line_base_orig = floatval($row['line_subtotal']);
		$line_tax_orig = floatval($row['line_subtotal_tax']);
		
		$line_base = floatval($row['line_total']);
		$line_tax = floatval($row['line_tax']);

		$line_price_orig = $line_base_orig + $line_tax_orig;
		$line_price = $line_base + $line_tax;
		$line_base_disc = $line_base_orig - $line_base;
		$line_price_disc = $line_price_orig - $line_price;
		
		// Incorporado a partir de 2.5.2, 2.7.1, 2.6.2
		$unit_base = $qty? $line_base/$qty : 0.0;
		$unit_base_disc = $qty? $line_base_disc/$qty : 0.0;
		$unit_base_orig = $qty? $line_base_orig/$qty : 0.0;
		$unit_tax = $qty? $line_tax/$qty : 0.0;
		$unit_price_orig = $qty? $line_price_orig/$qty : 0.0;
		$unit_price = $qty? $line_price/$qty : 0.0;
		$unit_price_disc = $qty? $line_price_disc/$qty : 0.0;
		
		$discount_perc = $line_price? $line_price_disc/$line_price : 0.0;
		
		return array(
			'order' => intval($row['id_order']),
			// invoice => intval($row['id_order_invoice']),
			'ref' => $ref,
            'name' => $row['product_name'],
		    'size' => empty($row['size'])? '' : strtoupper(sanitize_title($row['size'])),
		    'color' => empty($row['color'])? '' : strtoupper(sanitize_title($row['color'])),

			'quantity' => $qty,
			'tax' => $tax,
				
			'unit_base_orig' => $unit_base_orig,
			'unit_base_disc' => $unit_base_disc,
			'discount_perc' => $discount_perc,
			// unit_base => unit_base_orig - unit_base_disc,
			// line_base = unit_base * quantity
			'unit_tax_imp' => $unit_tax, // unit_tax
				
			// Se usaban antes de 2.5.2, 2.7.1, 2.6.2 ¿Harán falta después? Los conservamos por compat con 2.X
			'price' => $unit_price_orig,
			'discount' => $unit_price_disc,
			'total' => $line_price, // line_price = unit_price * quantity
			'tax_imp' => $line_tax, // line_tax = unit_tax * quantity
		);
	}
}


class Fs2psSizeExtractor extends Fs2psExtractor
{

	protected function buildSql()
	{
		return '
			SELECT distinct pm.meta_value 
			FROM @DB_postmeta pm 
			where pm.meta_key like \'attribute_talla%\' or pm.meta_key like \'attribute_size%\'
			order by pm.meta_value
		';
	}
	
	protected function row2dto($row)
	{
		return array(
			'ref' => sanitize_title($row['meta_value']), // sanitize_title(
			'name' => $row['meta_value']
		);
	}
}

/*
class Fs2psColourExtractor extends Fs2psExtractor
{

	protected function buildSql()
	{
		return '
			SELECT distinct pm.meta_value
			FROM @DB_postmeta pm
			where pm.meta_key =\'attribute_colores\' 
			order by pm.meta_value
		';
	}

	protected function row2dto($row)
	{
		return array(
				'ref' => $row['meta_value'],
				'name' => $row['meta_value']
		);
	}
}
*/

class Fs2psProductExtractor extends Fs2psExtractor
{
	
	protected function buildSql()
	{
		return '
			SELECT 
			    p.ID,
			    p.post_date,
			    p.post_modified,
			    p.post_title,
				p.post_status,
			    -- p.post_content,
			    -- p.post_excerpt,
			    (select meta_value from @DB_postmeta where post_id=p.ID and meta_key=\'_regular_price\') as price,
				(select meta_value from @DB_postmeta where post_id=p.ID and meta_key=\'_sale_price\') as sale_price,
			    (select meta_value from @DB_postmeta where post_id=p.ID and meta_key=\'_sku\') as sku,
			    (select meta_value from @DB_postmeta where post_id=p.ID and meta_key=\'_manage_stock\') as manage_stock,
			    (select meta_value from @DB_postmeta where post_id=p.ID and meta_key=\'_stock\') as stock,
			    (select meta_value from @DB_postmeta where post_id=p.ID and meta_key=\'_tax_status\') as tax_status,
			    (select meta_value from @DB_postmeta where post_id=p.ID and meta_key=\'_tax_class\') as tax_class
			    -- (select meta_value from @DB_postmeta where post_id=p.ID and meta_key=\'_thumbnail_id\') as thumbnail_id,
				-- (select meta_value from @DB_postmeta where post_id=p.ID and meta_key=\'_product_image_gallery\') as product_image_gallery,
			    -- (select meta_value from @DB_postmeta where post_id=p.ID and meta_key=\'_product_attributes\') as product_attributes
			    -- weight, width, length, height
				-- TODO: categoria
			FROM 
			    @DB_posts p
			WHERE p.post_type=\'product\' -- and p.ID=134
			ORDER BY p.ID
		';
	}

	protected function row2dto($row)
	{
		if (empty($row['sku'])) {
			$row['sku'] = '*'.$row['ID'];
			Fs2psTools::dbUpdate('postmeta', array('meta_value'=>$row['sku']), array('post_id'=>$row['ID'], 'meta_key'=>'_sku'));
		}
		
		$dto = array(
			'ref' => $row['sku'],
			'name' => $row['post_title'],
			'created' => $row['post_date'],
			'updated' => $row['post_modified'],
			'enabled' => $row['post_status']=='publish',
		);
		
		if ($row['tax_status']=='taxable') {
			$dto['tax_class'] = $row['tax_class'];
		}
		
		$cfg = $this->task->cfg;
		if ($cfg->get('PRICES_INCLUDE_TAX')) {
			$dto['price'] = floatval($row['price']);
			if (!empty($row['sale_price'])) $dto['sale_price'] = floatval($row['sale_price']);
		} else {
			$dto['price_base'] = floatval($row['price']);
			if (!empty($row['sale_price'])) $dto['sale_price_base'] = floatval($row['sale_price']);
		}

		if ($row['manage_stock']=='yes') {
			$dto['stock'] = floatval($row['stock']);
		}
		
		return $dto;
	}
	
}


class Fs2psSizeColourCombinationExtractor extends Fs2psExtractor
{

	protected function buildSql()
	{
		return '
			SELECT
			    p.ID,
				pp.ID as parentID,
			    -- p.post_content,
			    -- p.post_excerpt,
			    (select meta_value from wp_postmeta where post_id=p.ID and meta_key like \'attribute_talla%\' or meta_key like \'attribute_size%\') as attribute_tallas,
			    (select meta_value from wp_postmeta where post_id=p.ID and meta_key=\'_regular_price\') as price,
				(select meta_value from wp_postmeta where post_id=p.ID and meta_key=\'_sale_price\') as sale_price,
			    (select meta_value from wp_postmeta where post_id=p.ID and meta_key=\'_sku\') as sku,
				(select meta_value from wp_postmeta where post_id=pp.ID and meta_key=\'_sku\') as parent_sku,
			    (select meta_value from wp_postmeta where post_id=p.ID and meta_key=\'_manage_stock\') as manage_stock,
			    (select meta_value from wp_postmeta where post_id=p.ID and meta_key=\'_stock\') as stock,
			    (select meta_value from wp_postmeta where post_id=p.ID and meta_key=\'_tax_status\') as tax_status,
			    (select meta_value from wp_postmeta where post_id=p.ID and meta_key=\'_tax_class\') as tax_class,
			    (select meta_value from wp_postmeta where post_id=pp.ID and meta_key=\'_tax_class\') as parent_tax_class
			    -- (select meta_value from wp_postmeta where post_id=p.ID and meta_key=\'_thumbnail_id\') as thumbnail_id,
			    -- (select meta_value from wp_postmeta where post_id=p.ID and meta_key=\'_product_image_gallery\') as product_image_gallery,
			    -- (select meta_value from wp_postmeta where post_id=p.ID and meta_key=\'_product_attributes\') as product_attributes
			    -- weight, width, length, height
			FROM
			    wp_posts p
			    inner join wp_posts pp on pp.ID=p.post_parent and p.post_status=\'publish\'
			WHERE p.post_type=\'product_variation\' -- and pp.ID=134
			ORDER BY pp.ID, p.ID
		';
	}

	protected function row2dto($row)
	{
		if (empty($row['parent_sku'])) {
			$row['parent_sku'] = '*'.$row['parentID'];
			Fs2psTools::dbUpdate('postmeta', array('meta_value'=>$row['parent_sku']), array('post_id'=>$row['parentID'], 'meta_key'=>'_sku'));
		}
		
		$attribute_tallas = empty($row['attribute_tallas'])? '' : sanitize_title($row['attribute_tallas']);
		if (empty($row['sku'])) {
			if (!empty($attribute_tallas)) {
				$row['sku'] = $row['parent_sku'].'_'.$attribute_tallas;
			} else {
				$row['sku'] = '*'.$row['ID'];
			}
			Fs2psTools::dbUpdate('postmeta', array('meta_value'=>$row['sku']), array('post_id'=>$row['ID'], 'meta_key'=>'_sku'));
		}

		$dto = array(
			'ref' => $row['sku'],
			'product' => $row['parent_sku'],
			'size' => $attribute_tallas,
		);

		if ($row['tax_status']=='taxable') {
			$dto['tax_class'] = $row['tax_class']=='parent'? $row['parent_tax_class'] : $row['tax_class'];
		}

		$cfg = $this->task->cfg;
		if ($cfg->get('PRICES_INCLUDE_TAX')) {
			$dto['price'] = floatval($row['price']);
			if (!empty($row['sale_price'])) $dto['sale_price'] = floatval($row['sale_price']);
		} else {
			$dto['price_base'] = floatval($row['price']);
			if (!empty($row['sale_price'])) $dto['sale_price_base'] = floatval($row['sale_price']);
		}

		if ($row['manage_stock']=='yes') {
			$dto['stock'] = floatval($row['stock']);
		}

		return $dto;
	}

}

