<?php
/**
* LICENCIA
*
* Este programa se propociona "tal cual", sin garantía de ningún tipo más allá del soporte
* pactado a la hora de adquirir el programa.
*
* En ningún caso los autores o titulares del copyright serán responsables de ninguna
* reclamación, daños u otras responsabilidades, ya sea en un litigio, agravio o de otro
* modo, que surja de o en conexión con el programa o el uso u otro tipo de acciones
* realizadas con el programa.
*
* Este programa no puede modificarse ni distribuirse sin el consentimiento expreso del autor.
*
*    @author    Carlos Fillol Sendra <festeweb@festeweb.com>
*    @copyright 2014 Fes-te web! - www.festeweb.com
*    @license   http://www.festeweb.com/static/licenses/fs2ps_1.1.0.txt
*/

include_once(dirname(__FILE__).'/Fs2psDto2RowMatcher.php');
include_once(dirname(__FILE__).'/Fs2psException.php');
include_once(dirname(__FILE__).'/Fs2psTools.php');


class Fs2psDto2TermMatcher extends Fs2psDto2RowMatcher
{
 
    protected $taxonomy = null;
    
    public function __construct($entity, $table, $taxonomy)
    {
        $this->taxonomy = $taxonomy;
        parent::__construct($entity, $table, null, null, true);
    }
    
    public function dtoIdToStr($dto_id)
    {
        return sanitize_title(parent::dtoIdToStr($dto_id));
    }
    
    public function _rowIdFromDtoId($dto_id)
    {
        //$term = get_term_by('slug', $this->dtoIdToStr($dto_id), $this->taxonomy);
        $terms = Fs2PsTools::wpErr(get_terms(array(
            'get'                    => 'all',
            'number'                 => 1,
            'taxonomy'               => $this->taxonomy,
            'update_term_meta_cache' => false,
            'orderby'                => 'none',
            'suppress_filter'        => true,
            'slug'                   => $this->dtoIdToStr($dto_id),
        )));
        if (empty($terms)) return null;
        $term = array_shift($terms);
        
        return $term? $term->term_id : null;
    }
    
}

class Fs2psDto2PostMatcher extends Fs2psDto2RowMatcher
{
    
    public function __construct($entity, $dto_id_fields=null, $persist=false)
    {
        parent::__construct($entity, 'posts', 'ID', $dto_id_fields, $persist);
    }
    
}

class Fs2psDto2RowRefMatcher extends Fs2psDto2PostMatcher
{

    public function __construct($entity, $dto_id_fields=null, $persist=false)
	{
	    parent::__construct($entity, $dto_id_fields, $persist);
	}
	
	public function _rowIdFromDtoId($dto_id)
	{
		$row_id = Fs2psTools::dbValue("
			SELECT post_id
			FROM 
				`@DB_postmeta` pm
				inner join `@DB_posts` p on p.ID=pm.post_id
			where pm.meta_key='_sku' and pm.meta_value= '".$this->dtoIdToStr($dto_id)."'
		");
		return $row_id? intval($row_id) : null;
	}

}

class Fs2psDto2RowPRefMatcher extends Fs2psDto2RowMatcher
{
    
    public function __construct($entity, $dto_id_fields=null, $persist=false)
    {
        parent::__construct($entity, 'posts', 'ID', $dto_id_fields, $persist);
    }
    
    /**
     * Sólo debe usarse esta función en caso de que dto tenga suficiente información
     * como en el momento de la creación/actualización.
     *
     * Si existe un producto con matching por pref -> matching (si hay varios se cogerá el primero)
     * En caso contrario se intenta el matching básico por ref (cod FS)
     */
    public function rowIdFromDto($dto)
    {
        $dto_id = $this->dtoId($dto);
        $row_id = $this->rowIdFromDtoIdForcedMatches($dto_id);
        if ($row_id) return $row_id;
        
        if (!empty($dto['pref']))
        {
            $dto_pref_sql = Fs2psTools::dbEscape($dto['pref']);
            $row_id = (int)Fs2psTools::dbValue("
    			SELECT post_id
    			FROM
    				`@DB_postmeta` pm
    				inner join `@DB_posts` p on p.ID=pm.post_id
    			where pm.meta_key='_sku' and pm.meta_value= '".$dto_pref_sql."'
    		");
            if ($row_id) return $row_id;
        }
        
        return $this->_rowIdFromDtoId($dto_id);
    }
    
}

class Fs2psWoocommerceAttributeTaxonomiesMatcher extends Fs2psDto2RowMatcher
{
    
    public function __construct($entity, $dto_id_fields=null, $persist=false)
    {
        parent::__construct($entity, 'woocommerce_attribute_taxonomies', null, $dto_id_fields, $persist);
    }
    
    public function attributeNameFromDtoId($dto_id) {
        $dto_id_str = $this->dtoIdToStr($dto_id);
        return $dto_id_str=='SIZES'? 'size': ($dto_id_str=='COLOURS'? 'color' : preg_replace('/s$/', '', strtolower($dto_id_str)));
    }
    
    public function _rowIdFromDtoId($dto_id)
    {
        $row_id = Fs2psTools::dbValue("
			SELECT attribute_id
			FROM `@DB_woocommerce_attribute_taxonomies`
			where attribute_name= '".$this->attributeNameFromDtoId($dto_id)."'
		");
        return $row_id!==null? intval($row_id) : null;
    }
    
}

