<?php
/**
 * LICENCIA
 *
 * Este programa se propociona "tal cual", sin garantía de ningún tipo más allá del soporte
 * pactado a la hora de adquirir el programa.
 *
 * En ningún caso los autores o titulares del copyright serán responsables de ninguna
 * reclamación, daños u otras responsabilidades, ya sea en un litigio, agravio o de otro
 * modo, que surja de o en conexión con el programa o el uso u otro tipo de acciones
 * realizadas con el programa.
 *
 * Este programa no puede modificarse ni distribuirse sin el consentimiento expreso del autor.
 *
 *    @author    Carlos Fillol Sendra <festeweb@festeweb.com>
 *    @copyright 2014 Fes-te web! - www.festeweb.com
 *    @license   http://www.festeweb.com/static/licenses/fs2ps_1.1.0.txt
 */

include_once(dirname(__FILE__).'/Fs2psTools.php');
include_once(dirname(__FILE__).'/Fs2psUpdater.php');
include_once(dirname(__FILE__).'/Fs2psMatchers.php');
include_once(dirname(__FILE__).'/Fs2psObjectModels.php');


class Fs2psTermUpdater extends Fs2psUpdater {
	
	protected $taxonomy = null;
	
	public function __construct($task, $name, $taxonomy=null, $matcher=null) {
	    parent::__construct($task, $name, $matcher==null? new Fs2psDto2TermMatcher($name, 'terms', $taxonomy) : $matcher );
		$this->taxonomy = $taxonomy;
	}
	
	protected function dto2row($dto, $idx, $exists, $oldRowId) {
		$row = array();

		if ($this->enable) {
			$row['is_visible'] = 1;
		} elseif (!$exists) {
			$row['is_visible'] = 0;
		}
		
		if (!$exists || !$this->noover_content) {
			if (isset($dto['name']))
			{
				$name = $dto['name'];
				$row['name'] = $name;
				$row['slug'] = $this->matcher->referenceFromDto($dto); // sanitize_title?
			}
		}
		return $row;
	}
	
	protected function insertOrUpdate($row, $exists, $oldRowId) {
		$result = null;
		
		if ($exists) {
			$result = Fs2PsTools::wpErr(
				wp_update_term($oldRowId, $this->taxonomy, $row));
		} else {
			$result = Fs2PsTools::wpErr(
				wp_insert_term($row['name'], $this->taxonomy, $row));
		}
		
		return $result['term_id'];
	}
}

class Fs2psCategoryUpdater extends Fs2psTermUpdater {
	
	public function __construct($task, $name) {
	    parent::__construct($task, $name, 'product_cat');
	}
	
	protected function reloadCfg() {
		parent::reloadCfg();
		$cfg = $this->task->cfg;
		$this->noover_parent = $cfg->get('NOOVER_CATEGORIES_PARENT', false);
	}
	
	protected function dto2row($dto, $idx, $exists, $oldRowId) {
		$row = parent::dto2row($dto, $idx, $exists, $oldRowId);
		if (!$exists || !$this->noover_parent) {
			$row['parent'] = !empty($dto['parent'])? $this->matcher->rowIdFromDtoId($dto['parent']) : '';
		}
		return $row;
	}
	
}

class Fs2psAttributeGroupUpdater extends Fs2psUpdater {
	
	public function __construct($task, $name) {
	    parent::__construct($task, $name, new Fs2psWoocommerceAttributeTaxonomiesMatcher('woocommerce_attribute_taxonomies'));
	}
	
	protected function dto2row($dto, $idx, $exists, $oldRowId) {
	    $matcher = $this->matcher;
		$row = array(
		    'attribute_name' => $matcher->attributeNameFromDtoId($matcher->dtoId($dto)),
			'attribute_type' => 'select',
			'attribute_orderby' => 'menu_order'
		);
		
		if ($this->enable) {
			$row['attribute_public'] = 1;
		} elseif (!$exists) {
			$row['attribute_public'] = 0;
		}
		
		if (!$exists || !$this->noover_content) {
	        if (isset($dto['name'])) {
		      $row['attribute_label'] = $dto['name'];
		    }
		}
		
		return $row;
	}
	
	protected function insertOrUpdate($row, $exists, $oldRowId) {
        global $wpdb;
        $id = wc_attribute_taxonomy_id_by_name($row['attribute_name']); // TODO cfillol: Esto debería hacerlo el matcher
        if ($id) {
            $wpdb->update($wpdb->prefix.'woocommerce_attribute_taxonomies', $row, array('attribute_id'=>$id));
        } else {
            // If the attribute does not exist, create it.
            $id = Fs2PsTools::wpErr(wc_create_attribute(
                array(
                    'name'         => $row['attribute_label'],
                    'slug'         => $row['attribute_name'],
                    'type'         => $row['attribute_type'],
                    'order_by'     => $row['attribute_orderby'],
                    'has_archives' => false,
                )
            ));
            
            // Register as taxonomy
            //
            // IMPORTANTE:
            // Las taxonomías no están disponibles porque WP las hace disponibles en el init. Estarán disponibles en la próxima request.
            // Por eso hay que crear grupos de atributos y dar valores a esos atributos en dos requests distintas.
            $tax_name = 'pa_'.$row['attribute_name'];
            register_taxonomy(
                $tax_name,
                apply_filters('woocommerce_taxonomy_objects_' .$tax_name, array('product') ),
                apply_filters('woocommerce_taxonomy_args_' . $tax_name, array(
                    'labels'       => array( 'name' => $row['attribute_label'], ),
                    'hierarchical' => true,
                    'show_ui'      => false,
                    'query_var'    => true,
                    'rewrite'      => false,
                ))
            );
          }
            
          return $id;
	}
}

class Fs2psAttributeUpdater extends Fs2psTermUpdater {
    public function __construct($task, $name) {
        parent::__construct($task, $name, 'pa_'.($name=='sizes'? 'size' : 'color'));
    }
}

class Fs2psSKUUpdater extends Fs2psUpdater {
	
	protected function dto2row($dto, $idx, $exists, $oldRowId) {
		$rate = $dto['iva'];
		$regular_price = isset($dto['price']) ? $dto['price'] : (isset($dto['prices'])? $dto['prices'][0]['price'] : 0);
		$discount = isset($dto['prices']) && isset($dto['prices'][0]['dis'])? $dto['prices'][0]['dis'] : 0;
		$sale_price = $discount>0? $regular_price - $discount : 0;
		$cfg = $this->task->cfg;
		if ($cfg->get('PRICES_INCLUDE_TAX')) {
			$regular_price = intval(($regular_price * (1+$rate) + 0.005) * 100)/100.0;
			$sale_price = intval(($sale_price * (1+$rate) + 0.005) * 100)/100.0;
		}

		$row = array(
			'post_status' => 'publish',
			'_visibility' => 'visible',
		    '_manage_stock' => 'yes', // Antes if (!$exists) ??!
		    '_sku' => $this->matcher->referenceFromDto($dto),
			'_stock' => isset($dto['stock']) ? $dto['stock'] : 0,
			'__regular_price' => $regular_price,
		);
		if ($discount) $row['__sale_price'] = $sale_price;
		
		return $row;
	}

	protected function insertOrUpdate($row, $exists, $oldRowId) {
		$post = array();
		foreach ($row as $key => $value) {
			if (preg_match("/^post_.*/", $key))
				$post[$key] = $value;
		}
		if ($exists) {
			$post['ID'] = $oldRowId;
			kses_remove_filters();
			Fs2PsTools::wpErr(wp_update_post($post, true));
			kses_init_filters();
			$id = $oldRowId;
		} else {
			$id = Fs2PsTools::wpErr(wp_insert_post($post, true));
		}

		foreach ($row as $key => $value) {
			if (preg_match("/^_[^_].*/", $key)) {
				update_post_meta( $id, $key, $value );
			}
		}
		
		// Mostrar 'Agotado'? Sólo si no hay stock y está habilitada la gestión de existencias
		$manage_stock = empty($row['_manage_stock'])? get_post_meta($id, '_manage_stock', true) : $row['_manage_stock'];
		$manage_stock = (!empty($manage_stock) && ($manage_stock[0]=='y' || $manage_stock[0]=='Y'));
		$stock_status = $manage_stock? ($row['_stock']>0? 'instock' : 'outofstock') : 'instock';
		update_post_meta($id, '_stock_status', $stock_status);
		
		if (empty($row['__regular_price'])) {
			// Producto/Combinación sin precio
			$delete_metas = array('_regular_price', '_sale_price','_sale_price_dates_from','_sale_price_dates_to');
			foreach ($delete_metas as &$meta) {
				delete_post_meta($id, $meta);
			}
			update_post_meta($id, '_price', '');
		} else {
			// Producto/Combinación con precio y con posible oferta
			update_post_meta($id, '_price', empty($row['__sale_price'])? $row['__regular_price'] : $row['__sale_price']);
			update_post_meta($id, '_regular_price', $row['__regular_price']);
			update_post_meta($id, '_sale_price', isset($row['__sale_price'])? $row['__sale_price'] : '');
			update_post_meta($id, '_sale_price_dates_from', '');
			update_post_meta($id, '_sale_price_dates_to', '');
		}
		
		return $id;
	}
	
}

class Fs2psProductUpdater extends Fs2psSKUUpdater {
	
	protected $categories_upd;
	protected $tax_class_by_rate;
	protected $dims = array('width', 'height', 'depth', 'weight');

	public static function createMatcher($task, $name) {
	    $dto_id_fields = array('ref');
	    $pm = $task->cfg->get('PRODUCTS_MATCHER');
	    $matcher_cls = $pm=='basic'? 'Fs2psDto2PostMatcher' : ($pm=='pref'? 'Fs2psDto2RowPRefMatcher' : 'Fs2psDto2RowRefMatcher');
	    return new $matcher_cls($name, $dto_id_fields, true);
	}
	
	public function __construct($task, $name) {
	    $matcher = self::createMatcher($task, $name);
	    parent::__construct($task, $name, $matcher);
		$this->categories_upd = $task->getUpdater('categories');
		$this->loadTaxRulesGroupByRate();
	}
	
	protected function loadTaxRulesGroupByRate() {
		$tax_rules = Fs2psTools::dbSelect('
			SELECT distinct tr.tax_rate_class, tr.tax_rate
			FROM @DB_woocommerce_tax_rates tr
		');
		$tax_class_by_rate = array();
		foreach ($tax_rules as $rule)
		{
			$rate = (string)((float)$rule['tax_rate']);
			$tax_class_by_rate[$rate] = $rule['tax_rate_class'];
		}
		$this->tax_class_by_rate = $tax_class_by_rate;
	}
	
	protected function dto2row($dto, $idx, $exists, $oldRowId) {
		$rate = (string)($dto['iva'] * 100);
		if (!isset($this->tax_class_by_rate[$rate]))
		{
		    $msg = 'No se definió en la tienda un IVA del '.$rate.'% (prod. "'.$this->matcher->dtoIdToStrFromDto($dto).'")';
			$task = $this->task;
			if ($task->stop_on_error) {
				throw new Fs2psException($msg);
			} else	{
				$this->task->log('ERROR: '.$msg);
				return null;
			}
		}
		
		$row = parent::dto2row($dto, $idx, $exists, $oldRowId);
		
		Fs2psTools::replaceObjOrArray($row, array(
			'post_type'     => 'product',
			'_tax_class' => $this->tax_class_by_rate[$rate],
			'_visibility' => $this->enable ? 'visible' : 'hidden',
		));
		
		if (!$exists || !$this->noover_content) {
			if (isset($dto['name'])) $row['post_title'] = $dto['name'];
			if (isset($dto['descrip'])) $row['post_excerpt'] = $dto['descrip'];
			if (isset($dto['longdescrip'])) $row['post_content'] = $dto['longdescrip'];
		}
		
		foreach ($this->dims as $dim) {
			if (isset($dto[$dim])) 
				$row['_'.$dim] = $dto[$dim];
		}
		
		if (isset($dto['categories']))
			$row['__categories'] = $this->categories_upd->refsToIds($dto['categories']);
		
		if (isset($dto['combinations']))
			$row['_combinations'] = $dto['combinations'];
			
		return $row;
	}

	protected function insertOrUpdate($row, $exists, $oldRowId) {
		$id = parent::insertOrUpdate($row, $exists, $oldRowId);
		
		if (!empty($row['__categories'])) {
			foreach ($row['__categories'] as $cat_id) {
				Fs2PsTools::wpErr(
					wp_set_object_terms( $id, $cat_id, 'product_cat', true)
				);
			}
		} 
		else {
			$msg = 'WARN: El producto "'.$row['_sku'].'" no se asoció con ninguna categoría';
			$this->task->log($msg);
		}
		
		// No es necesario. Se refresca automaticamente si cambia _price, _regular_price o _sales_price
		// CUIDADO! Si _price, _regular_price o _sales_price pasan de cualquier valor a '', no se refresca.
		// wc_delete_product_transients( $id ); // Necesario para refrescar rango de precios
		
		return $id;
	}
	
	protected function onInsertedOrUpdated($dto, $row, $inserted)
	{
	    parent::onInsertedOrUpdated($dto, $row, $inserted);
	    if (isset($row['_combinations'])) {
	        $combinations_upd = $this->task->getUpdater('combinations');
	        if (empty($row['_combinations'])) {
	            $combinations_upd->removeNotPresentCombis($this->matcher->rowId($row), $row['_combinations'], true);
	        } else {
	            $combinations_upd->process($row['_combinations']);
	        }
	    }
	}
	
}

class Fs2psSizeColourCombinationUpdater extends Fs2psSKUUpdater {

	protected $products_upd;
	protected $field2type = array(
		'size'=> 'size',
		'colour'=> 'color'
	);
	protected $attr_fields = array('size', 'colour');
	protected $attr_types = array('size', 'color');
	
	
	public static function createMatcher($task, $name) {
	    // USE_CREF está obsoleto. Usar pref en su lugar combinado con matcher basic ó pref
	    // Siempre debería cumplirse que $dto_id_fields == array('ref', 'size', 'colour')
	    $dto_id_fields = $task->cfg->get('USE_CREF', False)? array('cref') : array('ref', 'size', 'colour');
	    
	    $pm = $task->cfg->get('COMBINATIONS_MATCHER');
	    
	    $matcher_cls = $pm=='basic'? 'Fs2psDto2PostMatcher' : ($pm=='pref'? 'Fs2psDto2RowPRefMatcher' : 'Fs2psDto2RowRefMatcher');
	    return new $matcher_cls($name, $dto_id_fields, ($pm=='basic' || $pm=='pref'));
	}
	
	public function __construct($task, $name) {
	    $matcher = self::createMatcher($task, $name);
	    parent::__construct($task, $name, $matcher);
		$this->products_upd = $task->getUpdater('products');
	}

	protected function dto2row($dto, $idx, $exists, $oldRowId) {
		$row = parent::dto2row($dto, $idx, $exists, $oldRowId);
		
		$parent_id = $this->products_upd->matcher->rowIdFromDtoId($dto['ref']);
		$post_title = Fs2psTools::dbValue('select post_title from @DB_posts where ID='.$parent_id);
		if (!empty($dto['colour'])) $post_title .= ' - '.$dto['colour'];
		if (!empty($dto['size'])) $post_title .= ' - '.$dto['size']; 
		
		Fs2psTools::replaceObjOrArray($row, array(
			'post_title' => $post_title,
			'post_parent'   => $parent_id,
			'post_type'     => 'product_variation',
			'_tax_class' => 'parent',
			//'guid'          =>  home_url() . '/?product_variation='.$name, //product-91-variation
			// '_visibility' => 'visible',
				
			// Extra info to be used in insertOrUpdate
			'__size' => isset($dto['size']) ? sanitize_title($dto['size']) : null, // slug en minúscula
			'__colour' => isset($dto['colour']) ? sanitize_title($dto['colour']) : null,
		));
		
		return $row;
	}

	protected function insertOrUpdate($row, $exists, $oldRowId) {
		$id = parent::insertOrUpdate($row, $exists, $oldRowId);
		
		foreach ($this->attr_fields as $attr_field)  {
			$attr_type = $this->field2type[$attr_field];
			if (empty($row['__'.$attr_field])) {
				delete_post_meta($id,'attribute_pa_'.$attr_type);
			} else {
				$attr_value = $row['__'.$attr_field];
				update_post_meta($id, 'attribute_pa_'.$attr_type, $attr_value);
			}
		}
		
		return $id;
	}
	
	public function removeNotPresentCombis($parent_id, $combinations, $simulate_full_process=false)
	{
	    $all_combis = Fs2psTools::dbSelect('
			select p.ID from `@DB_posts` p
			where
				p.post_parent = '.(int)$parent_id.' and
				p.post_type=\'product_variation\'
		');
	    if (!empty($all_combis)) {
	        if ($simulate_full_process) {
	            $this->resetCounters($combinations);
	        }
	        
	        $matcher = $this->matcher;
	        $updated_combi_ids = array();
	        foreach ($combinations as $combi) {
	            $updated_combi_ids[] = $matcher->rowId($combi);
	        }
	        foreach ($all_combis as $combi) {
	            $combi_id = $matcher->rowId($combi);
	            if (!in_array($combi_id, $updated_combi_ids)) {
	                Fs2PsTools::wpErr(wp_delete_post($combi_id, true));
	                $this->ndeleted++;
	            }
	        }
	        
	        if ($simulate_full_process) {
	            $this->logProcess();
	        }
	    }
	}

	protected function sameGroup($row_a, $row_b) {
		return $row_a['post_parent'] == $row_b['post_parent'];
	}

	protected function onGroupUpdated($group_rows) {
		if (!empty($group_rows))
		{
			$parent_id = $group_rows[0]['post_parent'];
			Fs2PsTools::wpErr(wp_set_object_terms(
				$parent_id, 'variable', 'product_type'));
			
			// Fill selectable dropdowns
			$attr_values_by_type = array();
			foreach ($this->attr_fields as $attr_field) {
				$attr_type = $this->field2type[$attr_field];
				foreach ($group_rows as $row) {
					if (!empty($row['__'.$attr_field])) {
						if (!isset($attr_values_by_type[$attr_type])) {
							$attr_values_by_type[$attr_type] = array();
						}
						if (!in_array($row['__'.$attr_field], $attr_values_by_type[$attr_type])) {
							$attr_values_by_type[$attr_type][] = $row['__'.$attr_field];
						}
					}
				}
				
				if (!empty($attr_values_by_type[$attr_type])) {
					Fs2PsTools::wpErr(wp_set_object_terms(
						$parent_id, $attr_values_by_type[$attr_type], 'pa_'.$attr_type, true));
				}
			}
			
			// Save fs2ps attributes
			$product_attributes = get_post_meta($parent_id, '_product_attributes', true); //array();
			if ($product_attributes) { // Hacemos hueco al principio
				$posinc = sizeof($attr_values_by_type);
				foreach ($this->attr_types as $attr_type) {
					unset($product_attributes['pa_'.$attr_type]);
				}
				foreach ($product_attributes as $k => $v) {
					$pos = empty($v['position'])? 0 : $v['position'];  
					$v['position'] = $pos + $posinc;
				}
			} else $product_attributes = array(); 
			$position = 0;
			foreach ($attr_values_by_type as $attr_type => $attr_value) {
				$product_attributes['pa_'.$attr_type] = array(
					'name' => 'pa_'.$attr_type,
					'value' => '', // TODO falta valor escogido?: $attr_value
					'position' => (string)$position,
					'is_visible' => '1',
					'is_variation' => '1',
					'is_taxonomy' => '1'
				);
				$position++;
			}
			update_post_meta($parent_id, '_product_attributes', $product_attributes);
			
			// TODO cfillol: Set default combination if none
		}

		// Remove not uploaded combinations
		$this->removeNotPresentCombis($parent_id, $group_rows);
		
		// En el post padre no es necesario. Se refresca automaticamente si cambia _price, _regular_price o _sales_price
		// En las combinaciones, si es necesario para que se refresque el precio global (del padre)
		// TODO: Optimizar:
		// - Cargar y volver a guardar padre para que él solo compruebe si es necesario borrar transients?
		// - Ó Invocar delete transients sólo si cambia el precio de alguna combi
		wc_delete_product_transients( $parent_id ); // Necesario para refrescar rango de precios
	}

}
