<?php
/**
* LICENCIA
*
* Este programa se propociona "tal cual", sin garantía de ningún tipo más allá del soporte
* pactado a la hora de adquirir el programa.
*
* En ningún caso los autores o titulares del copyright serán responsables de ninguna
* reclamación, daños u otras responsabilidades, ya sea en un litigio, agravio o de otro
* modo, que surja de o en conexión con el programa o el uso u otro tipo de acciones
* realizadas con el programa.
*
* Este programa no puede modificarse ni distribuirse sin el consentimiento expreso del autor.
*
*    @author    Carlos Fillol Sendra <festeweb@festeweb.com>
*    @copyright 2014 Fes-te web! - www.festeweb.com
*    @license   http://www.festeweb.com/static/licenses/fs2ps_1.1.0.txt
*/

include_once(dirname(__FILE__).'/Fs2psTools.php');
include_once(dirname(__FILE__).'/Fs2psTask.php');
include_once(dirname(__FILE__).'/Fs2psUpdaters.php');

class Fs2psUploadProductImagesTask extends Fs2psUpdaterTask
{

	public function __construct($mng, $cmd)
	{
		parent::__construct('upload_product_images', $mng, $cmd);
		$this->addUpdater('products', 'Fs2psProductUpdater');
		$this->addUpdater('colours', 'Fs2psAttributeUpdater');
	}

	protected function error($msg)
	{
		if ($this->stop_on_error)
			throw new Fs2psException($msg);
		$this->log('ERROR: '.$msg);
	}
	
	protected function _execute($cmd)
	{
		// { op:'...', images: [ product: 'Z001', file_name: '...., data: ' ' ]}

		$products_udt = $this->getUpdater('products');
		
		$product_images = array();
		if (isset($cmd['product_images'])) $product_images = $cmd['product_images'];
		
		
		$last_id_product = 0;
		$images = null;
		
		foreach ($product_images as $dto)
		{
			$position = empty($dto['position'])? 1 : intval($dto['position']);
			if ($position!=1) continue; // XXX cfillol: De momento sólo trabajamos con 1 imagen
			
			$id_product = $products_udt->matcher->rowIdFromDtoId($dto['product']);
			if (!$id_product) {
				$this->error('El producto no existe: '.$dto['product']);
				continue;
			}
			
			if ($last_id_product!=$id_product) {
                $images = get_attached_media('image', $id_product);
			}
			
			// Creamos la imagen
			$upload = wp_upload_bits($dto['name'], null, Fs2psTools::base64DataDecode($dto['data']));
			$file_path = $upload['file'];
			$wp_filetype = wp_check_filetype($upload['file'], null);
			$attachment = array(
			    'post_mime_type' => $wp_filetype['type'],
			    'post_title' => sanitize_file_name($dto['name']),
			    'post_content' => '',
			    'post_status' => 'inherit'
			);
			$attach_id = wp_insert_attachment($attachment, $file_path, $id_product);
			require_once(ABSPATH . 'wp-admin/includes/image.php');
			$attach_data = wp_generate_attachment_metadata($attach_id, $file_path);
			wp_update_attachment_metadata($attach_id, $attach_data);
			if ($position==1) {
			    $old_attach_id = get_post_thumbnail_id($id_product);
			    if ($old_attach_id) wp_delete_attachment($old_attach_id, true);
			    set_post_thumbnail($id_product, $attach_id);
			} else {
			    if (len($images)>=$position-1) {
			        wp_delete_attachment($images[$position-2]->ID, true);
			    }
			}
				
			$last_id_product = $id_product;
		}
		
	}

}
